inherit "/std/Object";
#include "config.h"
#include "move.h"

#define MUD 0
#define STREAM 1

int listen_fd;
int conn_fd;
int sending;

void create()
{
  set("short", "a bug");
  set("id",  ({"bug"}) );
  set("long", "a bug.\n");
  enable_commands();
  sending = 0;
}

void init()
{
  add_action("connect","connect");
  add_action("disconnect","disconnect");
  add_action("listen","listen");
}

void catch_tell(string str)
{
  int ret;

  if(sending)
    ret = socket_write(conn_fd,str);
}

int connect(string str)
{
  int ret;

  if(!str) return(0);
  conn_fd = socket_create(MUD,"socket_shutdown");
  ret = socket_connect(conn_fd,str,"receive_data");
  if(ret < 0){
    write("Bug unable to connect.\n");
    return(1);
  }
  write("Bug has connected.\n");
  sending = 1;
  return(1);
}

int disconnect(string str)
{
  int ret;

  sending = 0;
  ret = socket_close(conn_fd);
  if(ret < 0){
    write("Bug unable to disconnect.\n");
    return(1);
  }
  write("Bug has disconnected.\n");
  return(1);
}

int listen(string str)
{
  int number;
  int ret;

  if(!str) return(0);
  if(sscanf(str,"%d",number) < 1){
    write("listen: need number as argument.\n");
    return(1);
  }
  listen_fd = socket_create(MUD,"socket_shutdown");
  socket_bind(listen_fd,number);
  ret = socket_listen(listen_fd,"new_conn");
  if(ret < 0){
    write("Bug unable to listen.\n");
    return(1);
  }
  write("Bug is listening for peers on port " + number + ".\n");
  return(1);
}

void receive_data(int rec_fd,string mess)
{
  object hearer;

  write("receive_data: fd = " + rec_fd + " message = " + mess + "\n");
  hearer = environment(this_object());
  if(userp(hearer))
    tell_object(hearer,"bug says: " + mess);
  else
    tell_room(hearer,"bug says: " + mess,this_object());
}

void new_conn()
{
  int ret;
  object hearer;

  write("socket_accept: conn_fd = " + conn_fd + "\n");
  conn_fd = socket_accept(listen_fd,"receive_data");
  sending = 1;
  hearer = environment(this_object());
  ret = socket_close(listen_fd);
  if(ret < 0){
    if(userp(hearer))
      tell_object(hearer,"Bug tried to stop listening and failed.\n");
    else
      tell_room(hearer,"Bug tried to stop listening and failed.\n",this_object());
    return;
  }
  if(userp(hearer))
    tell_object(hearer,"Bug has connected.\n");
  else
    tell_room(hearer,"Bug has connected.\n",this_object());
  return;
}

void socket_shutdown(int fd)
{
  int ret;
  object hearer;

  write("socket_shutdown: fd = " + fd + "\n");
  if(fd == conn_fd){
    hearer = environment(this_object());
    sending = 0;
    if(userp(hearer))
      tell_object(hearer,"Bug has disconnected.\n");
    else
      tell_room(hearer,"Bug has disconnected.\n",this_object());
    return;
  }
}

int move(mixed dest)
{
  object ob;

  if (stringp(dest)) {
    call_other(dest, "??");
    ob = find_object(dest);
  } else
    ob = dest;

  if (!dest->receive_objects())
    return MOVE_NOT_ALLOWED;

  if ((int)dest->exit(this_object()))
    return MOVE_NOT_ALLOWED;

  move_object(this_object(), ob);
  return MOVE_OK;
}
