//domain.c
// This is the domain daemon for the TMI-2 mudlib.
// It's loosely based on some earlier code written by Sulam (April 12, 1992) and later
// modified by Jubal for multiguilding (Apr12/92).
// The present incarnation, as a daemon, was done by Buddha@TMI-2 on 1/14/93


#include <uid.h>
#include <domains.h>
#include <daemons.h>
#include <mudlib.h>

inherit DAEMON;

// okay, this can be called with several intentions:
// 1. pass just an object to it, and it will assume the object is a link, and tell you the 
// primary domain of that object.
// 2. add a second string arguement to either method 1 or 2, and it will tell you if the
// player is a member of the domain specified by the second arguement.
// The return value, will be either 0, or the name of the domain the user is a 
// member of.
 
varargs string
query_domain(object link, string domain)
{
	mapping domains, entry;
	string level;

	if(!link || !objectp(link))  return 0;
 
	domains = link->query("domains");
	if (!domains || !keys(domains)) return 0;  // default case, for a player
	if(!domain)  domain = domains["primary"];
	if(domain) entry = domains[domain];
	if(entry) level = entry["level"];
	if( ! level || member_array(level, LEVEL_LIST) == -1) return 0;
	return domain;
}

varargs string
query_domain_sponsor(object link, string domain)
{
	mapping domains, entry;

	domains = link->query("domains");
	if( !domains || !keys(domains)) return 0;
	if(!domain) domain = domains["primary"];
	if( member_array( domain, DOMAIN_LIST ) < 0) return 0;
	entry = domains[domain];
	return ( entry ? entry["sponsor"] : 0 );
}

 varargs string
query_domain_level(object link, string domain)
{
	mapping domains, entry;

	if(!link)  return LEVEL_LIST[0];
 
	domains = link->query("domains");
	if (!domains || !keys(domains)) return LEVEL_LIST[0];
	if (!domain) domain = domains["primary"];
	entry = domains[domain];
	return (entry ? entry["level"] : LEVEL_LIST[0]);
}

nomask int
set_domain(object link, string domain, string lev, string sponsor )
{
	mapping domains, rec;

	if (geteuid(previous_object() ) != ROOT_UID) return 0;
	domains = link->query("domains");
	if (nullp(domains) || !keys(domains)) domains = ([ ]);
	if (member_array(domain, DOMAIN_LIST ) < 0) return 0;
	if (member_array( lev, LEVEL_LIST ) < 0) return 0;

	/* remove from domain? */
	if( lev == LEVEL_LIST[1] && domain != 0 )
	{
		map_delete(domains, domain);
                link->set("domains", domains);
		return 1;
	}

	/* promote or add user to domain */
	rec = ([ "level" : lev, "sponsor" : sponsor ]);
	domains[domain] = rec;
	if( lev != LEVEL_LIST[1] && ! domains[ "primary" ] )
		domains[ "primary" ] = domain;
	link->set("domains", domains);
	return 1;
}

nomask int
set_primary_domain(object link, string d)
{
	mapping domains;

	if (geteuid(previous_object()) != ROOT_UID) return 0;
	domains = (mapping)link->query("domains");
	if (!domains || !keys(domains) || !domains[d]) return 0;
	if ( member_array(d, DOMAIN_LIST ) < 0) return 0;
	if (domains[d]["level"] == LEVEL_LIST[1]) return 0;
	domains["primary"] = d;
	return 1;
}

nomask string
query_primary_domain(object link)
{
	mapping domains;
	domains = (mapping)link->query("domains");
	if (!domains || !mapp(domains)) return 0;
	return domains["primary"];
}

/* EOF */
