 
//	File  	:  /adm/daemons/logind.c
//	Creator	:  Sulam@TMI  (12-13-91)
//	Updated	:  Sulam@TMI  (3-29-92)  Added real_name prompting
//		   Sulam@TMI  (4-10-92)  Added game locking
//		   Buddha@TMI  (9-4-92)  Added news system and
//		    expanded the login process
//
//	Overhauled by Buddha@TMI (12-92) to support a separate creation
//	module and to serve as a login daemon
//
//	Updated	:  Watcher@TMI  (2-9-93)  Moved news display to nes
//		    news daemon, and inserted check for users stuck
//		    in login limbo
//		   Watcher@TMI  (2-22-93)  Added an optional userlist
//		    display during the welcome screen
//		   Watcher@TMI  (2-23-93)  Streamlined Guest login process,
//		    added banish name system, and upgraded a few other things.
//		   Watcher@TMI  (3-9-93)  Added login failure notification.
//		   Watcher@TMI  (4-7-93)  Added max players, WIZ_LOCK,
//		    ADMIN_LOCK, and shutdown entry check.
//		   Watcher@TMI  (4/15/93)  Added NO_REMOTE_LOGIN check option.
//		   Watcher@TMI  (4/29/93)  Added hibernation checks.

//	Much of the login process can be customized to suit your 
//	specific variety of mud and its atmosphere.  These defines
//	can be found, for the most part, in /include/login.h
 
#include <uid.h>
#include <priv.h>
#include <config.h>
#include <mudlib.h>
#include <login.h>
#include <daemons.h>
#include <login_macros.h>


//  Prototyping
 
void logon(object ob);
int load_banished();
string active_users();
static void get_name(string str, object ob);
static void get_password(string pass, object ob, int count);
static int check_password(string pass, object ob);
static void choice(string choice, object ob, string name);
static void try_again(object ob);
static void register_site();
static void exec_old_copy(string s, object ob);
static void login_new_copy(object ob);
static void enter_world(object ob);
static void check_email(object user);

static mixed *banish;

void create() {  seteuid(ROOT_UID);  load_banished();  }

// This is called when a new user logs in.
 
void logon(object ob) {
   string local_host, pattern;
   int a1, a2, a3, a4;
 
    if (base_name(previous_object()) != CONNECTION)
	return;
 
#ifdef NO_REMOTE_LOGINS

   local_host = (string)NAME_SERVER->get_host_name(mud_name());

   if(!local_host || local_host == "" ||
      sscanf(local_host, "%d.%d.%d.%d", a1, a2, a3, a4) != 4) {
     write("\n  Could not resolve local network ip number from mud name " +
	   "server.\n  An accessible ip number must be available for user " +
	   "validation check.\n\n");
     ob->remove_user();
   return; }
 
   else  pattern = a1 + "." + a2 + ".%d.%d";
 
   if(!query_ip_number(ob) ||
      sscanf(query_ip_number(ob), pattern, a1, a2) != 2) {
      write("\n\t" + capitalize(mud_name()) + " is presently only accepting "+
	    " local logins.\n\tEnquires can be directed to " + ADMIN_EMAIL +
	    ".\n\n");
      ob->remove_user();
   return; }
 
#endif

    write(LOGIN_MSG);
 
#ifdef USER_LIST
    write(wrap("Current users: " + active_users()) + "\n");
#endif
 
    write(LOGIN_PROMPT);
    input_to("get_name", 2, ob);
 
return; }

static void get_name(string str, object ob) {
   string tmp, tmp1, tmp2;
   int bad_name, loop, i, sd_time;

  //  Enable ADMIN_LOCK in /include/login.h with a reason for lockout
  //  if you wish to allow only Admin onto the mud.
 
#ifdef ADMIN_LOCK
    if (!member_group(str, "admin")) {
	write(
	      "\n\n\n" +"		" + mud_name()
	      + " Is Temporarily Closed for Repairs.\n\n"
	      + "		Reason for Closure:\n"
	      + "		" + ADMIN_LOCK + "\n\n"
	      + "		Sorry for the inconvenience,\n"
	      + "			-The Management\n\n\n");
	ob->remove_user();
	return;
    }
#endif

 
  //  If a maximum number of users is defined in /include/login.h, check
  //  to make sure we haven't exceeded that number.  If so, and the user
  //  is not an Admin, tell them to try again later.
 
#ifdef MAX_USERS
 
   if(!member_group(str, "admin")) {
 
	if(sizeof(users()) > MAX_USERS) {
	write("\n\n  Sorry, " + capitalize(mud_name()) + " is full " +
	      "presently ... Please try again shortly.\n\n");
 	ob->remove_user();
	return; }
   }
#endif

  //  If NO_SHUTDOWN_LOGIN is defined and a non-admin tries to login 
  //  during an ongoing shutdown, they will be informed of the shutdown
  //  and when the mud is likely to be back up.

#ifdef NO_SHUTDOWN_LOGIN

   sd_time = (int)SHUTDOWN_D->query_shutdown();
   if(sd_time)  sd_time = (sd_time - time()) / 60;
 
   if(sd_time && sd_time < 5 && !member_group(str, "admin")) {
   write("\n\n  Sorry, a mud shutdown is currently in progress.\n  The mud "+
	 "should be back up in " + (sd_time + 3) + " minutes.\n" +
	 "  Hope to see you then!!\n\n");
   ob->remove_user();
   return; 
   }
#endif
 
   if (str == "") {
	try_again(ob);
	return;
    }
 
    if (strlen(str) > 10) {
	write("Sorry, your name can't have more than 10 characters.\n");
	write(LOGIN_PROMPT);
	input_to("get_name", 2, ob);
	return;
    }
 
    str = lower_case(str);
 
    for (i = 0; i < strlen(str); i++) {
	if (str[i] < 'a' || str[i] > 'z') {
	    write("Sorry, your name can only have letters. (a-z)\n");
	    write("Please enter a new name: ");
	    input_to("get_name", 2, ob);
	    return;
	}
    }
 
    // Does the user even exist?
 
    if(!file_exists(user_data_file(ob, str) + ".o")) {
	write("\n");
 
#ifdef BANISHED_NAMES
 
   if(BANISHED_NAMES) {

   for(loop=0; loop<sizeof(banish); loop++) {

        if(str == banish[loop]) {  bad_name = 1;  break;  }

        else if(sscanf(banish[loop],"*%s*", tmp) == 1) {
		if(sscanf(str, "%s" + tmp + "%s", tmp1, tmp2) == 2) {
                bad_name= 1;  break; }
        }

        else if(sscanf(banish[loop], "%s*", tmp) == 1) {
		if(sscanf(str, tmp + "%s", tmp1) == 1) {
                bad_name = 1; break; }
        }

   }

   if(bad_name) {
   write("Sorry, that character name is restricted.\n\n");
   write(LOGIN_PROMPT);
   input_to("get_name", 2, ob);
   return; }

   }
 
#endif /* BANISHED_NAMES */
 
#ifdef REGISTER_MSG
	write(REGISTER_MSG);
	call_out("remove_copy", 1, ob);  // Give a delay so message is seen
	return;
#else
 
	write("Is \"" + capitalize(str)
	      + "\" really the name you wish to use? (y or n): ");
	input_to("choice", 2, ob, str);
	return;
#endif
    }
 
    seteuid(str);
    export_uid(ob);
    seteuid(getuid());
    ob->SET_NAME(str);
 

   //  Enable WIZ_lOCK in /include/login.h if you wish to only allow
   //  wizards to login to the mud.  If you define a reason in WIZ_LOCK
   //  it will be displayed before the user is dumped.

#ifdef WIZ_LOCK
    if(!ob->query("wizard")) {
        write("\n\n\n" + "\t" + capitalize(mud_name()) + " is presently " +
              "closed to players.\n\n");

        if(WIZ_LOCK)  write( WIZ_LOCK );

        write("\n\n");

    ob->remove_user();
    return; }

#endif
 
    //	If it is the Guest char, you don't need the password
 
    if(str == "guest") { get_password("", ob,1); return; }
 
    input_to("get_password", 3, ob, 1);
    write(PASSWORD_PROMPT);
    return;
}

static void remove_copy(object ob) {  if(ob)  ob->remove();  }
 
static void get_password(string pass, object ob, int count) {
    object body;
    int hibernate;

    write("\n");
 
    if (!check_password(pass, ob) && (string)ob->query("name") != "guest") {
	count += 1;
	write("Sorry, that password is incorrect.\n");
	if (count > 3) {
	    write("\nYou have taken too many tries.\n");
	    ob->set("passwd_fail", ({ query_ip_name(ob), time() }) );
	    ob->remove();
	}
	input_to("get_password", 3, ob, count);
	write("Please reenter your password: ");
	return;
    }
 
    //	This code checks to see if the user is in hibernation.
 
   hibernate = ob->query("hibernate");
 
   if(hibernate && time() < hibernate) {
    write("\nYour character is in hibernation until " + ctime(hibernate) +
	  ".\nYou will be allowed to login again after that date. See you"+
	  " then!\n");
    ob->remove_user();
    return; }
 
   else if(hibernate)  ob->set("hibernate", 0);
 
    body = find_player((string) ob->NAME);
 
    //	If user is stuck in login limbo ... dest old copy
 
    if(body && !environment(body))  body->remove();
 
    if (body) {
	ob->SET_BODY_OB(body);
	if (interactive(body)) {
 
#ifdef ONE_GUEST
	    if((string)ob->query("name") == "guest") {
	    write("Sorry, there is already a Guest character active.\n" +
		  "Please try again shortly, or login as a new character.\n\n");
	    call_out("remove_copy",1, this_player());
 	    return; }
#else
	    if((string)ob->query("name") == "guest") {
	    exec_old_copy("n", ob);
	    return; }
#endif
 
	    write("\nYour other copy is still interactive.\n");
	    write("Do you want to throw it out? (y/n): ");
	    input_to("exec_old_copy", 2, ob);
	    return;
	}
 
#ifdef EXEC_COPY
   log_file(EXEC_COPY, capitalize((string)ob->NAME) + " :\texec copy from " +
	    query_ip_address(ob) + " [" + extract(ctime(time()),4,15) + "]\n");
#endif
 
	if (ob->connect())
        {
            body->restart_heart();
	} else {
	    write("Failed to reconnect...\n");
	    ob->remove();
        }
	return;
    }
    login_new_copy(ob);
}

void login_new_copy(object ob) {
 
    if (!ob->restore_body()) {
	write("For some reason, your body could not be restored.\n" +
	"You might want to send email to " + ADMIN_EMAIL + " about this\n" +
	      "or log in with another name to alert the admins.\n");
	ob->remove();
	return;
    }
    if (ob->connect())
	enter_world(ob);
    else {
	write("Login failed... Sorry... \n");
	ob->remove();
    }
}

void exec_old_copy(string s, object user) {
    object tmp, link;
 
	if (member_array(s, ({ "y", "Y", "Yes", "yes" }) ) == -1) {
	if(!wizardp((object)user->BODY_OB) && 
	   (string)user->query("name") != "guest") {
	    write("Then come back another time.\n");
	    user->remove();
	    return;
	}
 
#ifdef NEW_COPY
   log_file(NEW_COPY, capitalize((string)user->NAME) + " :\tNew copy from " +
	    query_ip_name(user) + " [" + extract(ctime(time()),4,15) + "]\n");
#endif
 
	login_new_copy(user);
	return;
	}
	link = user->BODY_OB->query_link();	// Old connection object
 
#ifdef FORCE_EXEC
   log_file(FORCE_EXEC, capitalize((string)user->NAME) + " :\tForce exec " +
	"from " + query_ip_name(user) + " [ " + extract(ctime(time()),4,15) +
	"]\n");
#endif 

	tell_object(user->BODY_OB,
	 "\nYour character has been displaced by someone from " +
	 query_ip_name(user) + ".\n");
	tmp = new(OBJECT);
	// exec them into any old object
	exec(tmp, user->BODY_OB);
	tmp->remove();
	if (user->connect())
	    write("Reconnected.\n");
	else
	    write("Reconnection failed.\n");
 
	link->remove();
	return;
}

static void enter_world(object user) {
    mixed *bad_pass;
 
    seteuid(geteuid(user));
    export_uid(user->BODY_OB);
    seteuid(getuid());
 
    bad_pass = (mixed *)user->query("passwd_fail");

    if(bad_pass) {
    tell_object(this_player(), "\nWARNING: Login failure " +
		ctime(bad_pass[1]) + " from " + bad_pass[0] + ".\n\n");
    user->set("passwd_fail", 0);
    }
 
    check_email(user);
    user->BODY_OB->setup();
}

static int
check_password(string pass, object ob)
{
    string              password;

    if (!ob->restore()) {
	write("Failed to restore login object.\n");
	ob->remove();
	return 0;
    }
    password = ob->PASS;
    if (password != crypt(pass, password))
	return 0;
    else
	return 1;
}


static void
choice(string choice, object user, string name)
{
    write("\n");
    if (lower_case(choice) == "n") {
	write("Ok, please reenter it then: ");
	input_to("get_name", 2, user);
	return;
    } else if (choice == "y" || choice == "Y") {
	//Site banning code is from Dainia@DreamShadow
	  if (BANISH_D->check_mail_site(query_ip_number())) {
	    write("\n\n        Your site has been placed on a registration by the\n"
		  + "        admins of " + mud_name()
		  + ".  This means that you must mail\n"
		  + "                " + ADMIN_EMAIL + "\n"
		  + "        and request a character.\n"
		  + "\n"
		  + "                        --The Management\n");
	    user->remove();
	    return;
	}
	seteuid(name);
	export_uid(user);
	seteuid(getuid());
	user->SET_NAME(name);
	NEWUSER_D->create_new_user(user);
	return;
    }
    write("Bad choice, please type y or n: ");
    input_to("choice", user, name);
    return;
}

static void
try_again(object ob)
{
    write("Sorry, that isn't a valid entry.\n");
    ob->remove();
}


void
check_email(object user)
{
    mapping             mail_stat;
    int		toread;

    mail_stat = (mapping) MAILER_D->mail_status(user->NAME);
    toread = mail_stat["unread"];
    if (toread)
        printf("\nYou have %d new piece%s of mail!\n\n",
            toread, (toread == 1 ? "" : "s"));
}
 
//  If online user display is selected, this function displays
//  all online, visible users to those in the login process.
 
string active_users() {
   mixed *who;
   string output;
   int loop;
 
   who = users();

   who = filter_array(who, "filter_invis", this_object());
 
   if(!who || !sizeof(who))  return "None.";
 
   who = map_array(who, "switch_name", this_object());
 
   if(sizeof(who) == 1)  output = who[0] + ".";
   else output = implode(who[0..sizeof(who)-2], ", ") + ", and " +
	    who[sizeof(who)-1] + ".";
 
return output; }
 
static int filter_invis(object who) {
   if(!who || !environment(who)) return 0;	// No logon's in display
   if((int)who->query("npc"))  return 0;	// No monsters in display
   return visible(who);  }
 
//  Switch user object pointer for name of user
 
static string switch_name(object who) {
   return (string)who->query("cap_name");  }

//  These functions parse and load banished names

int load_banished() {
   string *tmp, file;
   int loop;

   banish = ({ });			//  Initialize banish array
 
   file = read_file(BANISHED_NAMES);
   tmp = explode(file,"\n");
  
   for(loop=0; loop<sizeof(tmp); loop++)
	if(tmp[loop][0] == '#' || tmp[loop][0] == '\n' || tmp[loop] == "")
	  continue;
   	else  banish += ({ tmp[loop] });
 
return 1; }
 
int query_banished(string str) {
  return (member_array(str, banish) != -1); }
 
