 
// File : /adm/daemons/newuserd.c
//
// This is the character creation module.
// This daemon is called from logind.c if a player enters and requests an
// unused name. It then sets up the new user as per the user's preferences.
// Begun by Buddha, sometime in September 1992.
//
// Mobydick added race, skills, and stats, 10-11-92.
// Extended by Buddha in December 1992 and January 1993
//
// Psyche@TMI-2 (93/01/27) added acceptance of first-letter input
//	when choosing gender and race.
//
// Streamlined and option addition added by Watcher@TMI (2/23/93)
//
// Watcher@TMI (4/15/93) added AUTO_WIZHOOD option.
 
#include <logs.h>
#include <uid.h>
#include <priv.h>
#include <mudlib.h>
#include <config.h>
#include <login.h>
#include <login_macros.h>


static void set_skills(object player);
static void set_stats(object player);


void create() {
   seteuid(ROOT_UID);
}

void create_new_user(object user) {
   object body;

   if (geteuid(previous_object()) != ROOT_UID) return;
   user->SET_BODY(USER_OB); // let's assume a standard login
   body = new(USER_OB);
 
   body->set("name", (string)user->NAME, MASTER_ONLY);
   body->set ("cap_name", capitalize((string)user->NAME), MASTER_ONLY) ;
   body->set("snoopable", 1, MASTER_ONLY);
 
   user->SET_BODY_OB(body);
 
   cat(NPLAYER_INTRO);
 
   write("Please enter a unique password for your character: ");
   input_to("new_pass", 3, user);
   return;
}

static void new_pass(string pass, object user) {
   write("\n");
   if( strlen(pass) < 5 ) {
      write("I'm sorry, your password must be at least 5 characters.\n");
      write("Please change your password: ");
      input_to("new_pass", 3, user);
      return;
   }
   write("Please reenter your password to confirm: ");
   input_to("new_pass2", 3, pass, user);
   return;
}

static void new_pass2(string pass2, string pass, object user) {
   write("\n");
   if( pass == pass2 ) {
      user->SET_PASS(crypt(pass2, 0));
      write("\nYour gender can be male, female, neuter, or hermaphrodite.\n" +
         "Please enter your gender: ");
      input_to("new_gender", 2, user, (object)user->BODY_OB);
      return;
   }
   else {
      write("\nSorry, the passwords have to match.\n");
      write("Please enter your character's password: ");
      input_to("new_pass", 3, user);
      return;
   }
}

static void new_gender(string g, object user, object body) {
   write("\n");
   if (!g || member_array(g,
	({"male", "female", "neuter", "hermaphrodite", "m", "f",
	"n", "h" })) == -1) {
      write("You must chose male, female, neuter, or hermaphrodite: ");
      input_to("new_gender", user, body);
      return;
   }

/* Psyche@TMI-2 (93/27/01): translates one-letter input to
	corresponding full word */

	switch(g)
	{
		case "male":
		case "m": g = "male";
			break;
		case "female":
		case "f": g = "female";
			break;
		case "neuter":
		case "n": g = "neuter";
			break;
		case "hermaphrodite":
		case "h": g = "hermaphrodite";
			break;
		default: g = "";
			break;
	}
   body->set("gender", g, READ_ONLY);
   write ("Your race can be human, elf, dwarf, gnome, or orc.\n") ;
   write ("Please enter your race: ") ;
   input_to("new_race", user, body) ;
   return ;
}

static void new_race(string r, object user, object body) {
   if (!r || member_array(r,
		({ "elf", "dwarf", "gnome", "human", "orc", "e", "d",
		"g", "h", "o" })) == -1) {
	write("You must choose human, elf, dwarf, gnome, or orc.\n");
	input_to("new_race", user, body);
	return;
   }
	switch(r)
	{
		case "h": r = "human";
			break;
		case "e": r = "elf";
			break;
		case "d": r = "dwarf";
			break;
		case "g": r = "gnome";
			break;
		case "o": r = "orc";
			break;
		default: r = r;
			break;
	}
   body->set("race",r) ;
   write("\nPlease enter your email address (user@host): ");
   input_to("new_email", user, body);
   return;
}

static void new_email(string e, object user, object body) {
   string foo, fii;
   
   write("\n");
   if( sscanf(e, "%s@%s", foo, fii) != 2 ) {
      write("Sorry, it has to be in the form user@host\n");
      write("Please reenter your email address: ");
      input_to("new_email", user, body);
      return;
   }
   user->SET_EMAIL(e);
   write("Please enter your real name: ");
   input_to("get_real_name", user, body);
   return;
}

static void get_real_name(string rn, object user, object body) {
 
   write("\n");
 
   if(!rn || rn == "")  rn = "???";
 
   user->SET_RNAME(rn);
 
   // ok, that's all we need to know from them... let's get them connected.
   // the stats could be rolled, etc, but that's not my job to code -- buddha
 
   set_stats(body);
   set_skills(body);
 
   seteuid(geteuid(user));
   export_uid(body);
   seteuid(getuid());
   user->connect();
 
   cat(NPLAYER_NEWS);
 
#ifdef NEW_USER
   log_file(NEW_USER, capitalize((string)user->NAME) + " was created on " +
	    extract(ctime(time()), 4, 15) + ".\n") ;
#endif
 
//  If AUTO_WIZHOOD is defined in /include/config.h, then any newly created
//  users will automatically be granted wizard status, and given the PATH
//  given by the define. This would be handy on places like TMI where
//  wizard bits are freely given.

#ifdef AUTO_WIZHOOD

   user->set("wizard", 1);
   if(AUTO_WIZHOOD)  user->set("PATH", AUTO_WIZHOOD);

   write("\t[You have been granted automatic wizard status]\n");

#endif
 
   body->setup();
   user->save_data();
   body->save_data();
}


int clean_up() {
   destruct(this_object()); 
   return 1;
}

static void set_stats(object player) {

	int strength, intelligence, dexterity, constitution ;
	int hp, sp, total ;

	total = 0 ;
	while (total<50 || total > 70) {
		strength = 9+random(7)+random(7) ;
		intelligence = 9+random(7)+random(7) ;
		dexterity = 9+random(7)+random(7) ;
		constitution = 9+random(7)+random(7) ;
		total = strength + intelligence + dexterity + constitution ;
	}
	switch (player->query("race")) {
		case "elf" : {
			intelligence = intelligence + 3 ;
			constitution = constitution - 2 ;
			dexterity = dexterity + 1 ;
			strength = strength - 2 ;
			break ;
		}
		case "dwarf" : {
			intelligence = intelligence - 1 ;
			dexterity = dexterity - 2 ;
			constitution = constitution + 2 ;
			strength = strength + 1 ;
			break ;
		}
		case "gnome" : {
			strength = strength - 3 ;
			dexterity = dexterity + 3 ;
			constitution = constitution - 1 ;
			intelligence = intelligence + 1 ;
			break ;
		}
		case "orc" : {
			strength = strength + 3 ;
			constitution = constitution + 1 ;
			dexterity = dexterity - 1 ;
			intelligence = intelligence - 3 ;
		}
	}
	player->set("stat/strength", strength) ;
	player->set("stat/intelligence", intelligence) ;
	player->set("stat/dexterity", dexterity) ;
	player->set("stat/constitution", constitution) ;
	hp = 25 + constitution + random(21) ;
	sp = 45 + intelligence + random(21) ;
	player->set("hit_points", hp) ;
	player->set("max_hp", hp, LOCKED);
	player->set("spell_points", sp) ;
	player->set("max_sp", sp, LOCKED);
	return ;
}

static void set_skills(object player) {
	player->wipe_skills() ;
        player->set_skill("Thrusting weapons",0,"strength") ;
        player->set_skill("Cutting weapons",0,"strength") ;
        player->set_skill("Blunt weapons",0,"strength") ;
        player->set_skill("Parrying defense",0,"dexterity") ;
        player->set_skill("Shield defense",0,"dexterity") ;
        player->set_skill("Combat spells",0,"intelligence") ;
        player->set_skill("Healing spells",0,"intelligence") ;
        player->set_skill("Divinations spells",0,"intelligence") ;
	player->set_skill("Wilderness",0,"dexterity") ;
        player->set_skill("First aid",0,"dexterity") ;
        player->set_skill("Theft",0,"dexterity") ;
        player->set_skill("Stealth",0,"dexterity") ;
	return ;
}
