// The new, more efficient, improved ls command.
// Thanks to Huthar and Wayfarer for this one.
// Italian added the -ls options (18-11-92)
// Mobydick added the help 11-13-92, using the ls.command doc file.
// Hacked by Zeus to work a bit faster...by putting the output into
// an array of string, and displaying it at the end using more. 29/01/92
// Modified by Zeus to use get_dir(path, -1) thus increasing the overall
// speed by twofolds.
// Watcher rerouted the output through the mudlib's PAGER system (4/10/93).

#include <mudlib.h>

#define spaces "                                                                               "

inherit DAEMON ;

string *text;
int show_dots, show_long, line;

int long_ls(string path, mixed *files, int num);
int is_file(string file);
int remove_dots(string file);
void do_more(string *str);

int do_ls(string path) {
   int index,x,y,rows,cols,l,length,num,size,i;
   mixed *files;
   string *output;

   size = file_size(path);
   if (size > -1) {
      size /= 1000;
      if (size == 0) size = 1;
      files = explode(path,"/");
      path = files[sizeof(files)-1];
 
      if(show_long) 
	long_ls(implode(files[0..sizeof(files)-2], "/") + "/",
		({ ({ path }) }), 1);
      else write(size + " " + path + "\n");
      return 1;
   }
   if (path != "/") path += "/";
   files = get_dir(path, -1);
   if (!pointerp(files)) {
      size = file_size(path);
      if (size == -2) {
         return 1;
      }
      if (size == -1) {
         notify_fail("No such file or directory.\n");
         return 0;
      }
      write (size + " " + path + "\n");
      return 1;
   }
   if(!show_dots)
      files = filter_array(files, "remove_dots", this_object());
   num = sizeof(files);
   if(!num) {
      write("Directory is empty.\n");
      return 1;
   }
   if(show_long) long_ls(path, files, num);
   else {
   for (index = 0; index < num; index++) {
      i = strlen(files[index][0]);
      if (i > length) length = i;
   }
   length ++;
   if (length > 75) length = 75;
   cols = 80 / (length + 5);
   if (!cols) cols = 1; /* filenames can be huge in un*x */
   rows = num / cols;
   if (rows * cols < num) rows++;
   output = allocate(rows);
   for (y = 0; y < rows; y++) {
      output[y] = "";
      for (x = 0; x < cols; x++) {
         index = x * rows + y;
         if (index > num - 1) break;
         size = files[index][1];
         l = strlen(files[index][0]);
         switch(size) {
            case -2:
            output[y] += "   1 " + files[index][0] + "/";
            l++;
            break;
            case -1:
            output[y] += "   - " + files[index][0];
            break;
            default:
            size /= 1000;
            if (size == 0) size = 1;
            if (size < 10) output[y] += " ";
            if (size < 100) output[y] += " ";
            output[y] += " " + size + " " + files[index][0];
            break;
           }
         if (length - l > 0)
            output[y] += (spaces[1 .. (length - l)]);
        }
   }
   do_more(output);
   }
   return 1;
}

int long_ls(string path, mixed *files, int num) {
  int i;
  mixed *stats;
  string *output;

  output = allocate(num+2);
  output[0]="     Last change              Loaded           Size       Name";
  output[1]="--------------------   --------------------   ------   -----------";
  for(i=0; i<num; i++) {
    stats=stat(path+files[i][0]);
    if(sizeof(stats)>1) {
      if (stats[0]==0) stats[0]="";
      stats[0]=spaces[0..(8-strlen(stats[0]+" "))]+stats[0];
      stats[1]=ctime(stats[1]);
      if(stats[2]) stats[2]=ctime(stats[2]);
      else stats[2]="       < not loaded >   ";
      output[i+2] = sprintf("%s   %s %s   %s", stats[1][4..strlen(stats[1])],
        stats[2][4..strlen(stats[2])], stats[0], files[i][0]);
    }
    else output[i+2] = sprintf(spaces[0..46]+"<DIR>   "+files[i][0]);
  }
  do_more(output);
  return 1;
}


void do_more(string *str) {
   this_player()->more( str, 1, 1 );
}


int cmd_ls(string path) {
   string *Files,*Dirs;
   int i, num;
   string *wcfiles;
   string str1, str2;

   show_dots = 0;
   show_long = 0;
   seteuid(getuid(this_player()));
   if(path && sscanf(path,"-%s %s",str1,str2) == 2) {
      path = str2;
      switch(str1) {
        case "a" :
          show_dots = 1;
          break;
        case "l" :
          show_long = 1;
          break;
        case "al" :
        case "la" :
          show_long = 1;
          show_dots = 1;
          break;
        default :
          show_long = 0;
          show_dots = 0;
      }
   }
   else if(path && sscanf(path,"-%s",str1) == 1) {
      path = 0;
      switch(str1) {
        case "a" :
          show_dots = 1;
          break;
        case "l" :
          show_long = 1;
          break;
        case "al" :
        case "la" :
          show_long = 1;
          show_dots = 1;
          break;
        default :
          show_long = 0;
          show_dots = 0;
      }
   }
   wcfiles = (string *)this_player()->wild_card(path);
   if (!wcfiles || !sizeof(wcfiles)) {
	 write("No such file or directory.\n");
	 return 1;
   }
   num = sizeof(wcfiles);
   if(num == 1) {
      do_ls(wcfiles[0]);
      return 1;
   }
   Files = filter_array(wcfiles, "is_file", this_object());
   Dirs = wcfiles - Files;
   if(pointerp(Files) && sizeof(Files) > 0) {
      Files = path_file(Files[0]);
      do_ls(Files[0]);
   }
   if(pointerp(Dirs) && sizeof(Dirs) > 0) {
      for(i = 0; i < sizeof(Dirs); i++) {
         write(Dirs[i]+":\n");
         do_ls(Dirs[i]);
      }
   }
   return 1;
}

int is_file(string file) {
   return(file_size(file) >= 0);
}

int remove_dots(mixed *file) {
   if(file[0][0] == '.') return 0;
   return 1;
}

int help() {
	write (
"Give a list of all files in 'path'. If no argument is given, list the\n"+
"files in the current directory.\n"+
"'ls /' will give a list of all files in the root directory.\n") ;
	return 1 ;
}
