/*
// File:	_call.c
// Syntax:	int cmd_call( string a );
// Purpose:     allow wizards to call functions in objects
// History:
//              Idea from the scroll object tracer.
// Real Name:   Douglas Reay
// Mud Name:	Pallando
// Mudlib:      Ephemeral Dale
// Date V1.*:   1992 October
// Comments:	basic multi-reference tracer as bin commands
// Mudlib:      TMI-II
// Date V2.0:   1993 January 2nd
// Comments:    references now stored on player not as array in daemon.
// Date V2.1    1993 March 19th 
// Comments:    Changed ref commands over to get_objects()
*/
#include <mudlib.h>
#include "refs.h"

inherit DAEMON;
inherit REF_FUNCS;

#define SYNTAX "Syntax: call <object>;<function>;<arg>;<arg>\n"
#define FUNC_LIST ({ "query_skills", "query_nicknames", "query_aliases" })
#define TAB "\t"

mixed do_call( object ob, string func, mixed args );

cmd_call( string a )
{
  string str, *exp_a;
  mixed objs, funcs, args, tmp, ret, rets;
  object ob;
  int i, s, fi, fs;

  if( !a ) { notify_fail( SYNTAX ); return 0; }
  seteuid( geteuid( previous_object() ) );

  exp_a = explode( a, ";" );
  s = sizeof( exp_a );
  objs = exp_a[0];
  if( s > 1 ) funcs = exp_a[1];
  else funcs = FUNC_LIST;
  if( s == 3 ) args = ({ exp_a[2] });
  if( s > 3 ) args = exp_a[2..(s-1)];

  objs = resolv_ref( objs );
  if( objs == "users" ) objs = users();
  if( !pointerp( objs ) ) objs = ({ objs });
  tmp = ({ });
  s = sizeof( objs );
  for( i = 0 ; i < s ; i++ )
  {
    if( stringp( objs[i] ) ) ob = get_objects( objs[i],0,1 );
    else if( objectp( objs[i] ) ) ob = objs[i];
    if( !ob ) write( wrap( "Can't identify " + identify( objs[i] ) +
      " as an object." ) );
    else tmp += ({ ob });
  }
  if( !sizeof( tmp ) ) return 1;
  objs = tmp;

  funcs = resolv_ref( funcs );
  if( !pointerp( funcs ) ) funcs = ({ funcs });
  tmp = ({ });
  s = sizeof( funcs );
  for( i = 0 ; i < s ; i ++ )
  {
    if( stringp( funcs[i] ) ) tmp += ({ funcs[i] });
    else write( wrap( "Can't identify " + identify( funcs[i] )+
      " as a string." ) );
  }
  if( !sizeof( tmp ) ) return 1;
  funcs = tmp;

  if( pointerp( args ) && ( s = sizeof( args ) ) )
    for( i = 0 ; i < s ; i++ )
      args[i] = resolv_ref( resolv_str( args[i] ) );

  rets = ({ });
  s = sizeof( objs );
  fs = sizeof( funcs );
  for( i = 0 ; i < s ; i++ )
  {
    str = identify( objs[i] );
    for( fi = 0 ; fi < fs ; fi++ )
    {
      ret = do_call( objs[i], funcs[fi], args );
      if( ret[0] ) rets += ({ ret[0] });
      if( fs == 1 ) str = wrap( str + ret[1] );
      else str += (fi?"":"\n") + wrap( ret[1] );
    }
    write( str );
  }
  switch( sizeof( rets ) )
  {
    case 0: rets = 0; break;
    case 1: rets = rets[0];
  }
  set_ref( "default", rets );

  return 1;
}

do_call( object ob, string func, mixed args )
{
  mixed ret, err;
  int i, s;
  string str;
  object shad;
  function f;

  if( !function_exists( func, ob ) )
  {
    for( shad = shadow( ob, 0 ) ; shad ; shad = shadow( shad, 0 ) )
      if( function_exists( func, shad ) ) { ob = shad; break; }
    if( ob != shad ) return ({ 0, "- does not contain " + func + "()" });
  }
  str = "-> " + func;
  if( pointerp( args ) && ( s = sizeof( args ) ) )
  {
    str += "( ";
    for( i = 0 ; i < s ; i++ )
    {
      if( i ) str += ", ";
      str += identify( args[i] );
    }
    str += " )";
  } else str += "()";
    
/*
// how does one do this using function type variables?
  f = (: ob, func :);
  switch( s )
  {
    case 0: err=catch(ret=(*f));break;
    case 1: err=catch(ret=(*f)(args[0]));break;
    case 2: err=catch(ret=(*f)(args[0],args[1]));break;
    case 3: err=catch(ret=(*f)(args[0],args[1],args[2]));break;
    case 4: err=catch(ret=(*f)(args[0],args[1],args[2],args[3]));break;
    default: err=catch(ret=(*f)(args[0],args[1],args[2],args[3],args[4]));break;
  }
*/
  switch( s )
  {
    case 0: err=catch(ret=call_other(ob,func));break;
    case 1: err=catch(ret=call_other(ob,func,args[0]));break;
    case 2: err=catch(ret=call_other(ob,func,args[0],args[1]));break;
    case 3: err=catch(ret=call_other(ob,func,args[0],args[1],args[2]));break;
    default: err=catch(ret=call_other(ob,func,args[0],args[1],args[2],args[3]));
  }

  if( err ) return ({ 0, str + TAB + "= ERR(" + identify( err ) + ")" });
  return ({ ret, str + TAB + "= " + identify( ret ) });
}

int help()
{
  write( SYNTAX +
    "Effect: Calls the function <function> in object <object>,\n"+
    "passing as many arguments <arg> as you give.\n"+ 
    "If no function is specified a dump of the object is given.\n"+
    "<object> and <function> can be arrays (eg \"users\")\n"+
    "Any of them can be references (qv help refs)\n"+
  "" );
  return 1;
}

