 
//	File	:  /cmds/std/_look.c
//	Creator	:  Matt D. Robinson (Yakker/Regret @TMI)  1991
//	Updated	:  Help added by Brian (1/28/92)
//		   Rewritten by Adam Beeman (Buddha@TMI)  6/25/92
//		   Light handling by Steve Schmidt (Mobydick@TMI)  11/9/92
//		   Contents processing routines, light handling updated,
//		   and infravision by Glenn Ferguson (Watcher@TMI) 1/29/93
//
//	This code allows a user to "look" in a room, or at a specific
//	object in the user's inventory or environment.

#include <mudlib.h>

inherit DAEMON ;

// Contents display format flag:
//   Set as 0 = standard look, 1 = line by line processed,
//	    2 = processed linked together
 
#define FLAG	1
 
// prototypes...
string look_in_room(object room, int flag);
string look_at_item(string item, object user);
string look_in_item(string item, object user);
string look_at_item_in_item(string item, string box, object user); 
string convert_number(int num);
string process_contents(mixed *inv, int flag);
int content_match(object what);

mixed *number;
object act_ob, env, it;

// cmd_look(string) -- This function will 'look' in a room,
// and handle the case of whether it is just the room, or a
// separate object.

int cmd_look(string str) {

	int oldstatus ;
	string tmp, tmp2;
	mapping exits, doors;

	act_ob = this_player();
	env = environment(act_ob);

// Looking at something costs the player one attack.

	this_player()->block_attack(2) ;


	// Check to see if we have a string, and if we do, then see
	// if we are trying to look at a particular object.  If so
	// then get the object's description from the room.  Otherwise
	// ask the room for its own description.

	// are we looking at our immediate surroundings?
 
	if (!str || !stringp(str)) {
		// if we have no environment, give a default message.
		if (!env) {
			write("There is nothing to see here.\n");
			return 1;
		}
		write (look_in_room(env,(int)this_player()->query("force_to_look")));
		return 1;
	}
	// first see if we're looking at something in a container.
 
	if (sscanf(str, "at %s in %s", tmp, tmp2) == 2) {
 
		// yes, we're looking inside a container.
		// for now a quick hack since I don't know how to identify
		// a locked or closable container.
		// our assumption is that the standard container will have
		// a standard function that we can check to see if it's locked
		// or opaque. we are also assuming that we're using MudOS with
		// the new present() behavior, so we can check for numbers in
		// the id() function....
		write(look_at_item_in_item(tmp, tmp2, act_ob));
		return 1;
	}
	if (sscanf(str, "at %s", tmp) ) {
		write(look_at_item(tmp, act_ob));
		return 1;
	}
	if (sscanf(str, "in %s", tmp)) {
		write(look_in_item(tmp, act_ob));
		return 1;
	}
	if (environment(this_player())) {
		exits = env->query("exits");
		doors = env->query("doors") ;
		if (mapp(exits) && member_array(str, keys(exits)) != -1) {
		    if (doors && doors[str]) {
			if (doors[str]["status"] != "open") {
			    notify_fail ("The door is closed.\n") ;
			    return 0 ;
			}
		    }
		    env = find_object_or_load(exits[str]);
		    write(look_in_room(env,0));	
		    return 1;
		}
	}
 
	cmd_look("at " + str);
	return 1;
}

string living_description(object ob) {
        object *inv;
        int i, count;
	string tmp;
        string reg;
        string race ;
        string short;

        reg = ob->query("long");
        race = ob->query("race") ;
 
        if(!reg) reg = "";
 
	inv = all_inventory(ob);
 
	for(i=0; i<sizeof(inv); i++) {
	tmp = inv[i]->query("extra_look");
	if(tmp)  reg += tmp;
 	}

 
	if(race) {
	  if(this_player() == ob)
		reg += "You are " + article(race) + " " + race + ".\n";
	  else reg += (string)ob->query("cap_name") + " is " +
		article(race) + " " + race + ".\n" ;
	}
 
        count = 0;
        for (i = 0; i < sizeof(inv); i++)
                if (inv[i]->query("short"))
                        count++;
        if (count) {
		if(this_player() == ob) reg += "You are carrying:\n";
                else reg += capitalize(subjective((string)ob->query("gender"))) +
			" is carrying:\n";
                for (i=0;i < sizeof(inv); i++) {
                        short = (string)inv[i]->query("short");
                        if (short) {
                                if ((int)inv[i]->query("equipped")==1)
                                        short += " (equipped)" ;
                                if ((int)inv[i]->query("wielded")==1)
                                        short += " (wielded)" ;
                                short += ".\n";
                                reg += "  " + short;
                        }
                }
        }
	else if(this_player() == ob) reg += "You are empty handed.\n";
        else reg += capitalize(subjective((string)ob->query("gender"))) +
		" is empty handed.\n";
        return reg;
}

string item_description(string str, object user, int infra) {
	mapping items, doors;
	string tmp, ret, dir, foo;

	if(str == "me")  str = (string)this_player()->query("name");
 
        if(str == "door" || sscanf(str, "%s door", dir) == 1) {
 
	if(user && environment(user))
	   doors = environment(user)->query("doors");
	if(!doors || !sizeof(doors))
	   return "You don't see any doors here.\n";

	if(!dir) {
	   if(sizeof(doors) > 1)
	   return "Which door do you wish to look at?\n";

	   dir = keys(doors)[0];
 
	}

	if(infra)
	   return "That door gives off no heat pattern.\n";
 
	if (!doors[dir]) {
		return "There is no door in the "+dir+" direction.\n" ;
	}
	if(!doors[dir]["desc"])  tmp = "";
	else tmp = capitalize(doors[dir]["desc"]) + ".\n";

	if(doors[dir]["status"])
	tmp += "The " + dir + " door is " + doors[dir]["status"] + ".\n";

	if(!tmp || tmp == "")
	return "There is nothing special about the " + dir + " door.\n";
	else return tmp;
	}

	it = present(str, user);
	if (!it && env) it = present(str, env);
 
	if(it && (int)it->query("invisible") == 1 && !wizardp(this_player()))
	   return "I don't see that here.\n" ;
 
	if(it && (int)it->query("invisible") == 2 && 
	   !member_group(geteuid(this_player(1)), "admin"))
		return "I don't see that here.\n" ;
 
	if(infra) {
	if(living(it) || it->query("hot"))
	  return "Your infravision shows a bright heat pattern.\n";
   	else return "Your infravision shows no heat pattern.\n";
	}
 
	if (it) {
		if (living(it)) ret = living_description(it);
		else ret = (string)it->query("long");
	} else if (env) {
		// ok, this is the way to get things built into the room.
		// Assuming that we _can_ look at objects, check for room
   		// defined dummy items. 
		items = env->query("item_desc");
	        if (mapp(items) && items[str]) ret = items[str]; 
		if (!ret) { // now check for item functions...
			items = env->query("item_func");
			if (mapp(items) && items[str]) {
				(string)call_other(env, items[str]);
				ret = "" ;
				return ret;
			}
		}	
	}
	if (!ret) return "I don't see that here.\n";
	if (it) {
		if (it!=this_player())
		tell_object(it, user->query("cap_name") + " looks at you.\n");
		foo = it->query("cap_name") ;
		if (!foo) foo = it->query("short") ;
		if (!foo) foo = "something";
 
		if(it == this_player())
		  foo = objective((string)this_player()->query("gender")) +
			"self";
 
		if(living(it))
		say(user->query("cap_name") + " looks at " +
			 foo + ".\n", it);
		else say(user->query("cap_name") + " looks at " +
			 lower_case(foo) + ".\n", it);
	} else say(user->query("cap_name") + " looks at " + str + ".\n");
	return ret;	
}

string look_at_item (string str, object user) {

        int i;
        object *obs;

   //	Check if the user is blind.
 
   if((int)user->query("blind"))
	return "You are blind and cannot see.\n" ;
 
   //	Check to see if the user can see.
 
   if((int)user->query_vision())   return item_description(str, user, 0);
 
   //	Check to see if user has infravision capabilities
 
   if((int)user->query("infravision"))
	return item_description(str, user, 1);
 
   //	Guess there is no light here, then.
 
   return ("It is dark and you can't see a thing.\n");
}

string look_at_item_in_item(string at, string in, object user) {
	return "You strain to see, but cannot.\n";
	// yeah... not done yet.
}

string lit_room_description(object room, int infra, int flag) ;

varargs string look_in_room(object room, int flag) {

        int i;
        object *obs;

	act_ob = this_player();
        env = environment(act_ob);
   //	Check to see if the room can be seen.
 
   if((string)room->query("no_vision"))
	return (string)room->query("no_vision") ;
 
   //	Check to see if the user is blind
 
   if((int)act_ob->query("blind"))
	return "You are blind and cannot see anything.\n";
 
   //	Check to see if the user can see within the room.
 
   if((int)act_ob->query_vision())
	return lit_room_description(room, 0, flag);
 
   //	Check to see if the user has infravision
 
   if((int)act_ob->query("infravision"))
	return lit_room_description(room, 1, flag);
 
   //	Looks like there isn't any light here.
 
   return ("It is dark and you can't see a thing.\n") ;
}

string lit_room_description(object room, int infra, int flag) {

	string long, str ;
	object *contents, *live;
	int i ;
	string tmp ;
	mapping exits, doors ;
	string *dirs ;

// Check the player's "brief" property and get either the long or short
// description, as appropriate.  If infravision is being used, minimal
// details of the surroundings are given.
 
	if (infra)  long = "";
	else if (flag && this_player()->query("brief")) {
		long = room->query("short")+"\n" ;
	} else {
        	long = room->query("long");
	}
        if (!long) long = "";
	doors = room->query("doors") ;
	if (doors && !infra) {
		dirs = keys(doors) ;
		for (i=0;i<sizeof(doors);i++) {
			str = "The "+dirs[i]+" door is "+doors[dirs[i]]["status"]+".\n" ;
			long += str ;
		}
	}
	exits = room->query("exits");
        if (!exits)
	   if(infra)
		long += "You cannot detect any obvious exits.\n";
	   else long += "\tThere are no obvious exits.\n";
 
        else {
		dirs = keys(exits);
        	if(sizeof(dirs) == 1)
		  if(infra)
			long += "The only obvious exit you can detect is " +
			  dirs[0] + ".\n";
		   else long += "\tThe only obvious exit is " +
                        dirs[0] + ".\n";
 
        	else { 
		if (sizeof(dirs) == 2)
		   if(infra)
		   	long += "The only obvious exits you can detect are " +
				" and " + dirs[1] + ".\n";
		   else long += "\tObvious exits are " + dirs[0] +

				" and " + dirs[1] + ".\n" ;
 
		else
		   if(infra)
			long += "The only obvious exits you can detect are " +
				implode(dirs[0..sizeof(dirs)-2], ", ") +
				", and " + dirs[sizeof(dirs)-1] + ".\n";
		   else long += "\tObvious exits are: " +
                		implode(dirs[0..sizeof(dirs)-2], ", ") +
               			", and " + dirs[sizeof(dirs)-1] + ".\n";
		}
	}
 
	//  Process inventory of the room/object being examined
 
        contents = all_inventory(room);
 
	//  Check for infravision flag and living heat patterns
 
	if(infra) {
 
  	live = filter_array(contents, "filter_living", this_object());
 
	if(!sizeof(live))
	   long += "Your infravision shows no other heat patterns.\n";
	else if(sizeof(live) == 1)
	   long += "Your infravision shows only one heat pattern.\n";
	else long += "Your infravision shows " + convert_number(sizeof(live)) +
		     " heat patterns.\n";
	return wrap(long);  }

	if(!FLAG)
	for(i=0; i<sizeof(contents); i++) {
	tmp = (string)contents[i]->query("short");
	tmp = capitalize(tmp);
	if(contents[i] != this_player() && tmp && tmp != "")
		long += " " + tmp + "\n";
	}
 
	else long += process_contents(contents, FLAG);

        return long;
}

//  Filter out all objects without a possible heat pattern
 
int filter_living(object what) {
   if(what == this_player())  return 0;
   if(userp(what) && !interactive(what))  return 0;	// linkdead
   return (living(what) && !what->query("cold") || what->query("hot")); }
 
string look_in_item(string tmp, object user) {
	string tmp2;
	object *contents;
 
	it = present(tmp, user);
	if (!it && env) it = present(tmp, env);
	if (it) {
		// in our hands or in our environment.
		contents = all_inventory(it);
		if (contents) tmp2 = process_contents(contents, FLAG);
		if (!tmp2 || tmp2 == "")
			return "You don't see anything inside.\n";
		else return capitalize(tmp) + " contains: \n" + tmp2;
	}
	// this is where we could look for things in the room's properties.
	return "Look in what?\n";
}

string process_contents(mixed *contents, int flag) {
   mapping match;
   mixed *inv, *ninv;
   string tmp, ret;
   int loop;
 
   ret = "";
   inv = contents;
 
// Remove all objects with no short desc or command initiator
   inv = filter_array(inv, "clean_objects", this_object());

// Compile list of object with matching short descriptions

   match = unique_mapping(inv, "query", "short");
   ninv = keys( match );

   //	Loop through rooms contents and place them into a display string
 
   for(loop=0; loop<sizeof(ninv); loop++) {

/*
I do not understand why living objects ought to be handled seperately:
further, it was causing problems because when two monsters of the same
short desc where in a room, it looked like only one. So I commented the
special case out. Moby, 4-10-93.
   if(living(match[ninv[loop]][0]))  
	tmp = match[ninv[loop]][0]->query("short");
   else
*/
   if(sizeof(match[ninv[loop]]) > 1)
   tmp = convert_number(sizeof(match[ninv[loop]])) + " " +
	pluralize(ninv[loop]);
   else tmp = ninv[loop];

   if(flag == 1) {  ret += " " + capitalize(tmp) + "\n";  continue;  }
   else ret += tmp;

   if(loop == sizeof(ninv) - 1)  ret += ".";
   else if (loop == sizeof(ninv) - 2)  ret += ", and ";
   else ret += ", ";

   }

   if(flag == 2 && ret != "")  ret = wrap("You see " + ret);


return ret; }

//  Remove all invisible objects from the room's content's list
 
int clean_objects(object what) {
 
   if(((int)what->query("invisible") == 1 && !wizardp(this_player())) ||
      ((int)what->query("invisible") == 2 &&
	!member_group(geteuid(this_player(1)), "admin")))  return 0;
 
return (((string)what->query("short") && what != act_ob) ? 1 : 0);  }

// Compare each object's short desc with processing short desc
 
string content_match(object what) {  return (string)what->query("short");  }
 
// Convert numeric value to word format if below 21
 
string convert_number(int num) {
 
   if(!number)
   number = ({ "one", "two", "three", "four", "five", "six", "seven", "eight",
	       "nine", "ten", "eleven", "twelve", "thirteen", "fourteen",
	       "fifteen", "sixteen", "seventeen", "eighteen", "nineteen",
	       "twenty" });
 
    return ((num > 20) ? "many" : number[num - 1]); }
 
int 
help() {
  write("Command: look\nSyntax: look [at object]\n"+
        "\nIf no object is specified this command makes you look\n"+
        "around the room.  If an object is specified it lets you\n"+
        "examine the object more closely.\n");
  return 1;
}
