// File: /cmds/std/_party.c
// written by Zeus for TMI-2 and compatible mudlibs
// this command requires /std/user/team.c to be
// installed.

#include <mudlib.h>

#define NAME (string)this_player()->query("name")
#define CAPNAME (string)this_player()->query("cap_name")
#define LEADER (string)this_player()->query("leader")

inherit DAEMON;

int help();

int handle_form() {
     if((LEADER)) {
          notify_fail("You are already in a party.\n");
          return 0;
     }
     this_player()->init_team();
     write("You form a new party.\n");
     say(CAPNAME+" forms a new party.\n");
     return 1;
}

int handle_invite(string who) {
     object ob;

     if(!(LEADER)) {
          notify_fail("You are not in any party.\n");
          return 0;
     }
     if((LEADER) != (NAME)) {
          notify_fail("You are not leader of your party.\n");
          return 0;
     }
     if (environment(this_player()))
          ob = present(lower_case(who), environment(this_player()));
     if(!ob) {
          notify_fail("There is no one by that name here.\n");
          return 0;
     }
     if(ob->query("leader") && (string)ob->query("leader") == (LEADER)) {
          notify_fail(ob->query("cap_name")+" is already a member of your party.\n");
          return 0;
     }
     this_player()->invite_player(ob);
     return 1;
}

int handle_join(string who) {
     object ob;

     if((LEADER)) {
          notify_fail("You are already in a party.\n");
          return 0;
     }
     if (environment(this_player()))
          ob = present(lower_case(who), environment(this_player()));
     if(!ob) {
          notify_fail("There is no one by that name here.\n");
          return 0;
     }
     if(!ob->query("leader")) {
          notify_fail(ob->query("cap_name")+" is not in any party.\n");
          return 0;
     }
     if(ob->query("leader") != ob->query("name")) {
          notify_fail(ob->query("cap_name")+" is not leader of "+
          possessive((string)ob->query("cap_name"))+" party.\n");
          return 0;
     }
     ob->add_member(this_player());
     return 1;
}

int handle_remove(string who) {
     object ob;

     if(!(LEADER)) {
          notify_fail("You are not in any party.\n");
          return 0;
     }
     if((LEADER) != (NAME)) {
          notify_fail("You are not leader of your party.\n");
          return 0;
     }
     ob = find_living(lower_case(who));
     if(!ob) {
          notify_fail("There is no one by that name here.\n");
          return 0;
     }
     if(!ob->query("leader") || (string)ob->query("leader") != (LEADER)) {
          notify_fail(ob->query("cap_name")+" is not a member of your party.\n");
          return 0;
     }
     this_player()->disband_member(ob, 1);
     return 1;
}

int handle_leave() {
     object ob;

     if(!(LEADER)) {
          notify_fail("You are not in any party.\n");
          return 0;
     }
     if((LEADER) == (NAME)) {
          notify_fail("You are the leader of your party.\n");
          return 0;
     }
     ob = find_living(LEADER);
     if(!ob) {
          this_player()->set("leader", 0);
          notify_fail("Your leader is gone...PLEASE NOTIFY A WIZ.\n");
          return 0;
     }
     ob->disband_member(this_player(), 0);
     return 1;
}

int handle_members() {
     object ob, *mem;
     int i;

     if(!(LEADER)) {
          notify_fail("You are not in any party.\n");
          return 0;
     }
     if((LEADER) == (NAME)) ob = this_player();
     else ob = find_living(LEADER);
     if(!ob) {
          this_player()->set("leader", 0);
          notify_fail("Your leader is gone...PLEASE NOTIFY A WIZ.\n");
          return 0;
     }
     mem = ob->query_members();
     i = sizeof(mem)+1;
     write("Your party has "+i+" "+(i < 2 ? "member.\n" : "members.\n"));
     write("   "+ob->query("cap_name")+" [leader]\n");
     for(i = 0; i < sizeof(mem); i++) {
          if(!mem[i]) continue;
          write("   "+mem[i]->query("cap_name")+"\n");
     }
     return 1;
}

int handle_disband() {
     if(!(LEADER)) {
          notify_fail("You are not in any party.\n");
          return 0;
     }
     if((LEADER) != (NAME)) {
          notify_fail("You are not leader of your party.\n");
          return 0;
     }
     this_player()->disband_team();
     return 1;
}

int cmd_party(string str) {
     string foo, fii;

     if(!str && !(LEADER)) {
          notify_fail("usage: party <action> [<player>]\n");
          return 0;
     }
     if(!str && (LEADER)) {
          write("You are a member of "+capitalize(LEADER)+"'s party.\n");
          return 1;
     }
     if(sscanf(str, "%s %s", foo, fii) == 2) {
          switch(foo) {
               case "invite" : return handle_invite(fii); break;
               case "join"   : return handle_join(fii); break;
               case "remove" : return handle_remove(fii); break;
               default : return help();
          }
     } else if(sscanf(str, "%s", foo) == 1) {
          switch(foo) {
               case "form"    : return handle_form(); break;
               case "leave"   : return handle_leave(); break;
               case "members" : return handle_members(); break;
               case "disband" : return handle_disband(); break;
               default : return help();
          }
     }
     return help();
}

int help() {
     write("Usage: party form             non-members ONLY.\n"+
           "       party invite <player>  leader ONLY.\n"+
           "       party join <player>    non-members ONLY.\n"+
           "       party leave            members ONLY.\n"+
           "       party members          all members.\n"+
           "       party remove <player>  leader ONLY.\n"+
           "       party disband          leader ONLY.\n");
     write("This command allows players to form and manage small\n"+
           "parties. To join an existing party, you must be invited\n"+
           "by its leader. Once invited, you have 60 seconds to join\n"+
           "before your invitation expires. If you quit, die or go\n"+
           "netdead whilst in a party, your membership to that party\n"+
           "is automatically removed. If you are the leader of that\n"+
           "party, then the party will be disbanded.\n");
     return 1;
}
