 
/*
//	File	:  /cmds/std/_say.c
//	Creator	:  Watcher@TMI (01/29/93)
//
//	Standard say command with modification for loudness via race,
//	special speech formats, and anonymous voices in dark areas.
*/

#include <mudlib.h>

inherit DAEMON ;
 
//	Range of possible say verbs used in conjunction with user's
//	speech loudnesss.  If you don't want this feature, simply set
//	the VERBS array as ({ "say" })
 
#define VERBS	({ "whisper", "mumble", "sing", "say", "holler", \
		   "yell", "scream" })

//	VISUAL_CHECK = 1  :  Gives anonymous voices in a dark area
//	VISUAL_CHECK = 0  :  Gives speaker's name regardless of light levels
 
#define	VISUAL_CHECK	1
 
int compute_loudness(object target);
 
int cmd_say(string str) {
   mixed *who;
   object here;
   string name, verb, special;
   int loop, light, num;
 
   //	Block an attack from the queue as payment for the say use
 
   this_player()->block_attack(2) ;
 
   //	Save user's environment
 
   here = environment( previous_object() );
 
   if ((!str) || (str == " ")) {
      notify_fail("You mutter to yourself.\n");
      return 0;
   }
 
   //	Check to see if user has a special say verb
 
   special = (string) previous_object()->query("special_say");
 
   //	Get contents of the user's environment minus the user.

   if (here) 
      who = all_inventory( here );
   else
      who = ({ });
   who -= ({ previous_object() });
 
   //	Loop through and give the message to each member of the room
 
   for(loop=0; loop<sizeof(who); loop++) {
 
   //	Get calculated say type corresponding to user's and target's loudness
 
   num = compute_loudness( who[loop] );
 
   if(VISUAL_CHECK && !who[loop]->query_vision()) {
	if(wizardp(this_player()))  name = "Someone";
	else  name = "Somebody";
   }
   else name = (string) previous_object()->query( "cap_name" );
 
   if(special)
     message("say", wrap(name + " " + special + "s: " + str), who[loop]);
   else 
    message("say", wrap(name + " " + VERBS[num] + "s: " + str), who[loop]);
   }
 
   if(special)	write(wrap("You " + special + ": " + str));
   else		write(wrap("You say: " + str));
 
return 1;  }
 
//	This function calculates the correct say verb to use by
//	comparing the user's and target's loudness levels.

int compute_loudness(object target) {
   int loudness;
 
   loudness = ( ((int)previous_object()->query("loudness") -
	    (int)target->query("loudness")) + sizeof(VERBS)/2 );
 
   if(loudness < 0)  return 0;
   else if(loudness > sizeof(VERBS) - 1)  return (sizeof(VERBS) - 1);
 
return loudness; }
 
int help() {
 
   write("Usage: say <message>\n\n" +
	 "The say command sends <message> to everyone listening within the\n" +
	 "current room. How you sound to others depends on how relatively\n" +
	 "loud you sound to others. However, if you are invisible, no one\n" +
	 "can hear what you say.\n");
	return 1 ;
}
 
