// The "sponsor" command.
// Brought to you by Jubal.
// Notification and logging added by Buddha (4-24-92)
// Part of the TMI mudlib.
// Made it limit you to one LESS than your position by Sulam (4/25/92)
// Starting workroom corrected for new mudlib 9-24-92 by Prydain
// Revised to communicate with a modified Domains daemon by Buddha, and
// also allow you to promote someone who is not logged in at the
// moment (1/23/93). Mobydick helped a little with that.
// Changed to use the same generic workroom as home.c, Pallando 93-02-05

#include <mudlib.h>
#include <daemons.h>
#include <domains.h>
#include <login_macros.h>

inherit             DAEMON;

string              workroom;

int
cmd_sponsor(string arg)
{
    string              domain, target, under;
    string		fname ;
    object              tmp, myLink, targetLink;
    int                 myLev, targetLev;

    if( ! arg || sscanf( arg, "%s %s", target, domain ) < 2 )
    {
       notify_fail( "Usage: sponsor <name> <guild> [under]\n" );
        return 0;
    }
    sscanf( domain, "%s %s", domain, under );
    if (member_array(domain, DOMAIN_LIST) == -1) {
	notify_fail("No such domain.\n");
	return 0;
    }
    tmp = previous_object();
    if (!interactive(tmp))
	return 0;
    //no security hole here...
      myLink = (object) tmp->query_link();
    myLev = member_array(
		      (string) DOMAIN_D->query_domain_level(myLink, domain),
			 LEVEL_LIST);
 /* If the command giver is an admin...well, let them do whatever. */
    if (member_group(geteuid(myLink), "admin"))
	myLev = sizeof(LEVEL_LIST);
    if (myLev < MIN_SPONSOR_LEVEL) {
	notify_fail(
		    "You must be at least " +
		    LEVEL_LIST[MIN_SPONSOR_LEVEL] +
		    " level in the domain to sponsor someone.\n");
	return 0;
    }
 /*
  * Ask the finger daemon to get a link for us, so that if they are not on at
  * the time, we can still do this. 
  */
    targetLink = FINGER_D->restore_data(target);
    if (!targetLink) {
	write (target+".\n") ;
	notify_fail("No such user.\n");
	return 0;
    }
    targetLev = member_array(
		  (string) DOMAIN_D->query_domain_level(targetLink, domain),
			     LEVEL_LIST);
    if (targetLev >= myLev) {
	write((string) targetLink->CAP_NAME +
	      " is too high a level for you to sponsor.\n");
	return 1;
    } else if (targetLev < 1) {
	targetLev = ENTRY_LEVEL;
	if (DOMAIN_D->set_domain(targetLink, domain,
			LEVEL_LIST[targetLev], geteuid(myLink))) {
	    write( "Level set to " + LEVEL_LIST[ targetLev ] + " (" +
			( under ? under : getuid(this_player()) ) + ").\n" );
	    tell_object(targetLink, (string)this_player()->query("cap_name") +
			" has sponsored you to novice"+
                        " in the " + domain + " domain.\n");
	    log_file("sponsors", (string)this_player()->link_data("cap_name") +
		     " promoted " + (string)targetLink->link_data("cap_name") +
			" to " + LEVEL_LIST[targetLev] +
                        " in the " + domain + " domain.\n");
        }
	else
	{
		write( "Failed to set level.\n" );
		return 1;
	}

	fname = "/u/" + extract( target, 0, 0 ) + "/" + target;
	if( file_size( fname ) != -2 )
	{
		if( file_size( fname ) >= 0 )
		{
			write( "Directory name currently taken by file.\n" );
			write( "Unable to create directory.\n" );
			return 1;
		}
		mkdir( fname );
		write( "Created directory " + fname + "\n" );
	}
	fname += "/workroom.c";
	if( file_size( fname ) == -1 )
	{
                if( !workroom ) workroom = read_file( "/obj/workroom.c" );
                write_file( fname, workroom );
		write( "Created file " + fname + "\n" );
	     log_file("workrooms", "Created " + fname + " on " +
			ctime(time()) + ".\n");
	}
 /*
  * This will force a save, and if the user is not on, a remove() as well. 
  */
	targetLink->save_data();
        targetLink->clean_up();

	return 1 ;
    }
    targetLev++;
    if (DOMAIN_D->set_domain(targetLink, domain,
			     LEVEL_LIST[targetLev], geteuid(myLink))) {
	tmp = find_player(target);
	if (tmp)
	    tell_object(tmp, (string) myLink->CAP_NAME +
		    " has promoted you to " + LEVEL_LIST[targetLev] +
		    " in the " + domain + " domain.\n");
	    write("Level set to " + LEVEL_LIST[targetLev] + ".\n");
	    log_file("promotes", (string) myLink->CAP_NAME +
	    " promoted " + capitalize(target) + " to " + LEVEL_LIST[targetLev] +
		     " in the " + domain + " domain.\n");
    } else {
	write("Failed to set level.\n");
    }
 /*
  * This will force a save, and if the user is not on, a remove() as well. 
  */
    targetLink->clean_up();

    return 1;
}

int help() {

	write(
"Command: sponsor\n" +
"Syntax:  sponsor <name> <domain> [under]\n" +
"  The sponsor command is used to enroll and promote <name> in the\n" +
"domain <domain>.  You must be of at least initiate level in domain\n" +
"<domain> to use sponsor.  If <name> is not in domain <domain>, he/she\n" +
"will be placed in the domain at the novice level.  If <name> is\n" +
"already in the domain, then he/she will be promoted one level.  You\n" +
"can only promote people up to one level below yourself.  If <name>\n" +
"does not have a directory, then one will be created for him/her,\n" +
"and a basic workroom placed in it.  Please exercise some caution\n" +
"in who you promote to initiate level, as he/she will be able to add\n" +
"new directories with this command.\n" +
"  Special for sages and admins, the <under> parameter allows you to\n" +
"change who is the offical sponsor of <name>.  If <name> is not in\n" +
"<domain>, then he/she will be added, otherwise his/her level will\n" +
"be unchanged.\n" +
"  See also:  domains, demote\n" );
	return 1;
}


/* EOF */


