 
// File : /std/body.c
//
// This file contains the code common to all player bodies. It is inherited
// by ghost.c, the ghost body, and living.c, which is the file containing
// the code common to the monster body and the player body.
// This file was created 3-10-93 by Mobydick@TMI-2, in conjunction with
// Watcher@TMI-2 as part of creating the new bodies system, which in turn
// was made possible by the work on login.c done by Buddha@TMI-2 in
// January 1993.
//
// All of the functions in this file were in the original user.c found
// at TMI-2 before the bodies project was launched. Its documentation is
// as follows:
// Original Authors: Sulam and Buddha @ The Mud Institute
// Many other people have added to this as well.
// This file has a long revision history dating back to the
// hideous mudlib.n and is now probably not at all the same.
//
// This file is a part of the TMI-2 distribution mudlib.
// Please keep this header with any code within.

#include <config.h>
#include <mudlib.h>
#include <daemons.h>
#include <body.h>
#include <logs.h>
#include <priv.h>
#include <uid.h>
#include <move.h>

inherit "/std/body/attack.c" ;
inherit "/std/body/track.c" ;
inherit "/std/body/alias.c" ;
inherit "/std/body/wild_card.c" ;
inherit "/std/body/more.c" ;
 
static object link;

// Prototypes. The important one is the user/nonuser recognition function.

void save_data() ;
void basic_commands() ;
static int init_commands();
 
//  Connection link support functions

object query_link() { 
	return link;
}

void set_link(object ob) {
	if (geteuid(previous_object()) != ROOT_UID && 
	    base_name(previous_object()) != CONNECTION)  return;
	link = ob;
}

mixed link_data(string what) {
	if (!link) return 0;
	return (mixed)link->query(what);
}

//  User debug function 

void debug(string mess) {
#ifdef DEBUG
   write( mess );
#endif
}
 
//  This function switchs the user from its present body to
//  that of a ghost when they "die".
 
static object create_ghost() {
   object ghost, old;
 
   //	First confirm the body has a link

   if(!link)  return 0;
 
   ghost = new(GHOST);
 
   old = environment(this_object());
 
   link->set("dead", 1);
   link->set("tmp_body", ghost);
 
   if(!link->switch_body()) {
   write("Error in the ghost body transfer.  Please inform an Admin.\n");
   return 0; }
 
   ghost->set("name", link_data("name") );
   ghost->init_setup();
   ghost->setup_ghost();
   link = 0;

#ifdef GHOST_START_LOCATION
   if(catch(call_other(GHOST_START_LOCATION, "???")) ||
      catch(ghost->move(GHOST_START_LOCATION) != MOVE_OK))
        ghost->move(START);
#else
   ghost->move(environment());
#endif
 
   if(environment(ghost) != old) 
   tell_room(environment(ghost),
	"The ghost of " + (string)ghost->query("cap_name") + " shimmers " +
        "into sight.\n", ({ ghost }));
 
return ghost; }
 
// The force hook.

int force_me(string cmd) {
   string tmp, verb;
   int res;
 
   tmp = geteuid(previous_object());
 
   if(tmp != ROOT_UID && tmp != geteuid(this_object()))
      return 0;

   //  Process command for aliases/nicknames/etc

   if(interactive(this_object()))
        cmd = (string)this_object()->process_input(cmd);

   verb = explode(cmd, " ")[0];
 
   //	Check to make sure the force is not an illegal force. If so,
   //	block it and notify all the parties of the foul deed.

   if(BAD_FORCE_VERBS && member_array(verb, BAD_FORCE_VERBS) != -1) {
   tell_object(this_object(), (string)this_player()->query("cap_name") + 
       " using " + identify(previous_object()) + " tried to force you to " +
       cmd + ".\n");
   write("Illegal force attempt blocked and noted.\n");

#ifdef BAD_FORCE
   log_file(BAD_FORCE, wrap((string)this_player()->query("cap_name") +
	" using " + identify(previous_object()) + " tried to force " +
	(string)this_object()->query("cap_name") + " to " + cmd + " [" +
	extract(ctime(time()),4,15) + "]\n"));
#endif

   return 0; }
 
   res = command(cmd);
 
   return res;
}
 
//  This function initializes the body shell prior to transfer
//  of the user connection.
 
void init_setup() {
  
   //  Set object's living name for hash table

   set_living_name( query("name") );
 
   //	Set user's euid permissions
 
   seteuid( getuid(this_object()) );
 
   //	Start this user's heartbeat
 
   set_heart_beat(1);
 
   //	Set user object's id and ip properties
 
   set( "id", ({ query("name") }) );
   set( "ip", query_ip_name(this_object()) );
 
 
   //	Enable wizard status if user fulfills prerequisites
 
   if((int)query_link()->query("wizard"))
	enable_wizard();
 
   //	Enable basic user commands defined in specific body
 
   basic_commands();
 
   //	Initialize body's standard command catch
 
   init_commands();
 
   //	Setup user's autosave call system
 
#ifdef AUTOSAVE
   call_out("autosave_user", AUTOSAVE);
#endif
 

   //	Setup important body system presents.
 
   set("volume", 500) ;
   set ("capacity", 5000-(int)this_object()->coins_carried()) ;

   //   Reset the weapon and armor defaults

   set ("weapon1",0) ;
   set ("weapon2",0) ;
   set ("armor", ([]) ) ;

   //   Retrieve autoload requests

   this_object()->load_autoload_obj();
 
   //	Initialize the tsh system here so all bodies get it

   call_out("initialize_tsh", 0);
 
}
 
void enable_me() {
        if (geteuid(previous_object()) != ROOT_UID) return ;
        enable_wizard() ;
}

void receive_message(string class, string msg) {
   object shell;
   string foo ;

// Invisibility screen. This should catch all tells and says and so forth,
// and all soul commands, and most command messages.

   if(this_player() && this_object() != this_player()) {
 
      if((int)this_player()->query("invisible") == 1 &&
         !wizardp(this_object()))
	    return;
 
      if((int)this_player()->query("invisible") == 2 &&
	 !member_group(geteuid(this_object()), "admin"))
	    return;
    }

   //  If this_object() is a monster, pass the message so
   //  aggressive monsters can react to certain messages.
 
   if((int)query("npc"))  
	this_object()->relay_message(class, msg);

   //  Mudlib support for user environment shell
 
   shell = present("shell", this_object());

   if(shell && previous_object() != shell &&
      base_name(shell) + ".c" == (string) this_object()->query("shell"))
        shell->receive_message(class, msg);
   else receive(msg);
}

// This is a new apply in MudOS 0.9.16.18. If a wizard is snooping a
// player, then the snooped messages are passed to this function.
// We either pass it off to a mudlib shell, or receive it as normal.

void receive_snoop (string str) {
   object shell;
 
   //  Mudlib support for user environment shell

   shell = present("shell", this_object());

   if(shell && previous_object() != shell &&
      base_name(shell) + ".c" == (string) this_object()->query("shell"))
 	shell->receive_message("snoop", "% " + str[0..strlen(str)-2]);
 
   else receive("% " + str);
}

void net_dead() {
   if (userp(this_object())) save_data() ;
   save_data();
   say((string)this_object()->query("cap_name") + " has gone net-dead.\n");
   ANNOUNCE->announce_user(this_object(), 3);
   CMWHO_D->remove_user(this_object());
   set_heart_beat(0);
   log_file("ENTER", this_object()->query("cap_name") + ": net-dead\t\t" + 
      ctime(time()) + " from " + query("ip") + "\n");
   link->remove();
}
 
void
restart_heart() {
   say(query("cap_name")+" has reconnected.\n");
   write("Reconnected.\n");
   ANNOUNCE->announce_user(this_object(), 2);
   set_heart_beat(1);
}

string local_commands() { 
   mixed *cmds;
   int i;
   string result;
 
   if(geteuid(previous_object()) != ROOT_UID &&
      !member_group(geteuid(previous_object()), "admin"))
      return "You aren't authorized to check this information.\n";
 
   cmds = commands();
 
   if (!sizeof(cmds)) return "No commands available";
 
   result = "";
   while (i < sizeof(cmds)) {
      result += (cmds[i][0] + " ");
      i++;
   }
 
   return result + "\n";
}

mixed *user_commands() { return commands(); }

